#include "Utilities.h"

GLuint LoadQuads(GLuint* elementArrayBuffer, GLsizei* length)
{
    GLuint VBO;
    GLuint VAO;
    GLuint EAB;

    std::vector<glm::vec2> positions;

    // First quad
    positions.push_back(glm::vec2(-0.1f, -0.5f));
    positions.push_back(glm::vec2( 0.6f, -0.5f));
    positions.push_back(glm::vec2( 0.1f,  0.2f));
    positions.push_back(glm::vec2( 0.8f,  0.2f));

    // Second quad
    positions.push_back(glm::vec2(-0.7f, -0.1f));
    positions.push_back(glm::vec2(-0.4f,  0.1f));
    positions.push_back(glm::vec2(-0.7f,  0.6f));
    positions.push_back(glm::vec2(-0.4f,  0.7f));

    std::vector<glm::vec3> colors;

    // First quad
    colors.push_back(glm::vec3(1.0f, 0.0f, 0.0f));
    colors.push_back(glm::vec3(1.0f, 0.0f, 0.0f));
    colors.push_back(glm::vec3(1.0f, 0.0f, 0.0f));
    colors.push_back(glm::vec3(1.0f, 0.0f, 0.0f));

    // Second quad
    colors.push_back(glm::vec3(0.0f, 1.0f, 0.0f));
    colors.push_back(glm::vec3(0.0f, 1.0f, 0.0f));
    colors.push_back(glm::vec3(0.0f, 1.0f, 0.0f));
    colors.push_back(glm::vec3(0.0f, 1.0f, 0.0f));

    std::vector<GLshort> indices;

    // First quad
    indices.push_back(0);
    indices.push_back(1);
    indices.push_back(2);
    indices.push_back(1);
    indices.push_back(2);
    indices.push_back(3);

    // Second quad
    indices.push_back(4);
    indices.push_back(5);
    indices.push_back(6);
    indices.push_back(5);
    indices.push_back(6);
    indices.push_back(7);

    struct Vertex
    {
        glm::vec2 position;
        glm::vec3 color;
    };

    std::vector<Vertex> BufferData;

    for (int i = 0; i < positions.size(); i++)
    {
        BufferData.push_back({ positions[i], colors[i]});
    }

    glGenBuffers(1, &VBO);

    glBindBuffer(GL_ARRAY_BUFFER, VBO);
    glBufferData(GL_ARRAY_BUFFER, sizeof(Vertex) * BufferData.size(), BufferData.data(), GL_STATIC_DRAW);

    glGenVertexArrays(1, &VAO);
    glBindVertexArray(VAO);

    glVertexAttribPointer(0, 2, GL_FLOAT, GL_FALSE, sizeof(Vertex), (void*)0);
    glVertexAttribPointer(1, 3, GL_FLOAT, GL_FALSE, sizeof(Vertex), (void*)sizeof(glm::vec2));

    glGenBuffers(1, &EAB);
    glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, EAB);
    glBufferData(GL_ELEMENT_ARRAY_BUFFER, indices.size() * sizeof(GLushort), &indices[0], GL_STATIC_DRAW);

    glEnableVertexAttribArray(0);
    glEnableVertexAttribArray(1);

    *elementArrayBuffer = EAB;
    *length = indices.size();

    return VAO;
}